# LOCK-BEGIN[imports]: DON'T MODIFY
from podite import (
    AutoTagType,
    Enum,
    pod,
)

# LOCK-END


# LOCK-BEGIN[class(DerivativeError)]: DON'T MODIFY
@pod
class DerivativeError(Enum[AutoTagType]):
    ACCOUNT_ALREADY_INITIALIZED = None
    INVALID_SETTLEMENT_TIME = None
    INVALID_CREATION_TIME = None
    UNINITIALIZED_ACCOUNT = None
    INVALID_SEQUENCE_NUMBER = None
    UNSETTLED_ACCOUNTS = None
    INVALID_ORACLE_CONFIG = None
    NUMERICAL_OVERFLOW = None
    CANNOT_BE_DELETED = None
    CONTRACT_IS_EXPIRED = None
    INVALID_DATE = None
    INVALID_ACCOUNT = None
    # LOCK-END

    @classmethod
    def _to_bytes_partial(cls, buffer, obj, **kwargs):
        # to modify packing, change this method
        return super()._to_bytes_partial(buffer, obj, **kwargs)

    @classmethod
    def _from_bytes_partial(cls, buffer, **kwargs):
        # to modify unpacking, change this method
        return super()._from_bytes_partial(buffer, **kwargs)

    @classmethod
    def to_bytes(cls, obj, **kwargs):
        return cls.pack(obj, converter="bytes", **kwargs)

    @classmethod
    def from_bytes(cls, raw, **kwargs):
        return cls.unpack(raw, converter="bytes", **kwargs)



import json
import sys
print('importing dexterity (it takes a while)...', end='', flush=True)
from dexteritysdk.dex.sdk_context import SDKContext, SDKTrader
from dexteritysdk.utils.aob import Side
print('success')
from solana.rpc.api import Client
from solana.keypair import Keypair
from solana.publickey import PublicKey

def load_keypair(fp):
    with open(fp, "r") as f:
        kp = json.load(f)
    return Keypair(kp).from_secret_key(kp)

def main():
    network = "http://localhost:8899/"
    client = Client(network)

    print('loading keypairs...', end='', flush=True)
    keypair = load_keypair('../deploy_key.json')
    mpg_keypair = load_keypair('../target/deploy/mpg-keypair.json')
    print('success')

    print('connecting...', end='', flush=True)
    ctx = SDKContext.connect(
	client=client,
	market_product_group_key=mpg_keypair.public_key,
	payer=keypair,
	raise_on_error=True
    )
    print('success')

    print('creating new trg...', end='', flush=True)
    trader = ctx.register_trader(keypair)
    print('success (trg pubkey: {})'.format(trader.account))

    print('depositing 100k USDC...', end='', flush=True)
    trader.deposit(ctx, 100000)
    print('success')

    print('withdrawing 1k USDC...', end='', flush=True)
    trader.withdraw(ctx, 1000)
    print('success')

    print('checking balance...', end='', flush=True)
    trg, _ = trader.get_trader_risk_group()
    trg.cash_balance
    print('success... (cash balance: {})'.format(trg.cash_balance))

    print('checking balance...', end='', flush=True)
    trg, _ = trader.get_trader_risk_group()
    trg.cash_balance
    print('success... (cash balance: {})'.format(trg.cash_balance))

    print('listing products and placing and cancelling orders...')
    for product in ctx.products:
        print(f"placing order on {product.name}...", end='', flush=True)
        size, price = 1, 100
        order_summary = trader.place_order(ctx, product, Side.ASK, size, price)
        print('success')
        print(f"cancelling order on {product.name}...", end='', flush=True)
        trader.cancel(ctx, product, order_summary.order_id)
        print('success')
    print('success')

    

main()
